package main

import (
    "fmt"
    "math"
    "math/rand"
    "time"
)

type tentativo struct {
  s string
  gpg, gps int
}

/* Genera e restituisce una slice con tutte le sequenze di n caratteri fra 'A' e 'F' */
func sequenze(n int) []string {
  if n == 0 {
    return []string{""}
  }
  seq := sequenze(n - 1)   // Chiamata ricorsiva
  var risultato []string
  for _, s := range seq {
    for _, car := range []rune{'A', 'B', 'C', 'D', 'E', 'F'} {
      t := s + string(car)
      risultato = append(risultato, t)
    }
  }
  return risultato
}

/* Data una stringa magic (segreta) e una stringa attempt (il tentativo dell'avversario)
   restituisce quanti sono i caratteri giusti al posto giusto (gpg) e quanti sono giusti
   al posto sbagliato */
func check(magic, attempt string) (gpg int, gps int) {
  // Queste due mappe contano quante volte ogni runa compare nelle due stringhe
  // esclusi i caratteri giusti al posto giusto
  var magicMap, attemptMap map[byte]int
  magicMap = make(map[byte]int)
  attemptMap = make(map[byte]int)
  for i := 0; i < 5; i++ {
    if magic[i] == attempt[i] {
      gpg++
    } else {
      magicMap[magic[i]]++
      attemptMap[attempt[i]]++
    }
  }

  for r, occ := range magicMap {
    gps += int(math.Min(float64(occ), float64(attemptMap[r])))
  }
  return
}

/* Data una slice seq, costruisce e restituisce una slice contenente le sole
   sequenze di seq che soddisfano i tentativi fatti (contenuti nella slice tent) */
func filtraSequenze(seq []string, tent []tentativo) []string {
  var res []string
  for _, s := range seq {
    ok := true
    for _, t := range tent {
      gpg, gps := check(s, t.s)  // Potrebbe essere s proprio la stringa da indovinare?
      if gpg != t.gpg || gps != t.gps {
        ok = false
        break
      }
    }
    if ok {
      res = append(res, s)
    }
  }
  return res
}

func main() {
  var gpg, gps int
  var tentativi []tentativo

  rand.Seed(time.Now().UnixNano())

  tutte := sequenze(5)  // Un insieme di 6^5 sequenze
  for {
    alcune := filtraSequenze(tutte, tentativi)
    //fmt.Println("Sequenze possibili fra cui scegliere: ", alcune)
    t := alcune[rand.Intn(len(alcune))]
    fmt.Println(t)
    fmt.Scan(&gpg, &gps)
    if gpg == 5 {
      break
    }
    tentativi = append(tentativi, tentativo{t, gpg, gps})
    //fmt.Println("***", tentativi)
  }
  fmt.Println("Che bello, ho indovinato!")
}
